

from .cut_video import cut_video
from .cut_timeframes import cut_timeframes
from .concat_videos import concat_videos
from .support import (
    get_audio_codec,
    get_video_codec,
    get_audio_duration,
    get_video_duration
)


import random
import subprocess
import os.path


def get_video_duration(input_path):
    cmd = f"ffprobe -v error -select_streams v:0 -show_entries stream=duration -of default=noprint_wrappers=1:nokey=1 {input_path}"
    data = subprocess_call([x.strip() for x in cmd.split(' ') if x.strip()])
    data = [x for x in data.split('\n') if x.strip()][0].strip()
    try:
        return float(data)
    except:
        return 0

def get_audio_duration(input_path):
    cmd = f"ffprobe -v error -select_streams a:0 -show_entries stream=duration -of default=noprint_wrappers=1:nokey=1 {input_path}"
    data = subprocess_call([x.strip() for x in cmd.split(' ') if x.strip()])
    data = [x for x in data.split('\n') if x.strip()][0].strip()
    try:
        return float(data)
    except:
        return 0


def get_video_codec(input_path):
    cmd = f"ffprobe -v error -select_streams v:0 -show_entries stream=codec_name -of default=noprint_wrappers=1:nokey=1 {input_path}"
    data = subprocess_call([x.strip() for x in cmd.split(' ') if x.strip()])
    data = [x for x in data.split('\n') if x.strip()][0].strip()
    try:
        return data
    except:
        return 'libx264'

def get_audio_codec(input_path):
    cmd = f"ffprobe -v error -select_streams a:0 -show_entries stream=codec_name -of default=noprint_wrappers=1:nokey=1 {input_path}"
    data = subprocess_call([x.strip() for x in cmd.split(' ') if x.strip()])
    data = [x for x in data.split('\n') if x.strip()][0].strip()
    try:
        return data
    except:
        return 'aac'

        
class FfmpegError(Exception):
    def __init__(self, message, stdout=''):

        # Call the base class constructor with the parameters it needs
        super().__init__(f'{message}\n{stdout}\n')

        # Now for your custom code...
        self.stdout = stdout

def subprocess_call(cmd, verbose=True, errorprint=True):
    """ Executes the given subprocess command."""

    popen_params = {"stdout": subprocess.PIPE,
                    "stderr": subprocess.PIPE,
                    "stdin": subprocess.DEVNULL}
    # pretty_cmd = ' '.join(cmd)
    # print(f'executing {pretty_cmd}')
    proc = subprocess.Popen(cmd, **popen_params)

    out, err = proc.communicate()
    # print(out)
    # proc.stderr.close()

    if proc.returncode:
        raise FfmpegError(err.decode('utf8'), out.decode('utf8'), )

    del proc
    return out.decode('utf8')


class temporary_write:
        def __init__(self,  data, path=str(random.random())[3:]):
                self.path = os.path.abspath(path)
                self.data = data
                f = open(self.path, 'w')
                f.write(self.data)
                f.close()

        __enter__ = lambda self: self.path
        __exit__ = lambda self, a, b, c: os.remove(self.path)

def name_and_ext(path):
    path, ext = os.path.splitext(path)
    name = os.path.basename(path)
    return name, ext



import os.path
from .support import subprocess_call
from .align_audio_video import align_audio_video
from .support import name_and_ext, temporary_write, get_video_codec, get_audio_codec




def cut_video(in_path, out_path, start, end, encode=False, acodec=None, vcodec=None,):
    name, ext = name_and_ext(out_path)
    if end - start > 4 and not encode:
        with temporary_write('', path=f'disaligned_temporary_{name}{ext}') as temp_path:
            cmd = [
              "ffmpeg","-y",
              "-i", in_path,
              "-ss", "%0.3f" % start,
              "-to", "%0.3f" % end,
              "-map","0",
              "-c", "copy",
              "-shortest",
              # "-avoid_negative_ts", "1",
              temp_path
            ]
            subprocess_call(cmd)
            out_path = align_audio_video(temp_path, out_path)
            return out_path
    else:
        vcodec = vcodec or get_video_codec(in_path)
        acodec = acodec or get_audio_codec(in_path)
        cmd = [
          "ffmpeg","-y",
          "-i", in_path,
          "-ss", "%0.3f" % start,
          "-to", "%0.3f" % end,
          "-map","0",
          "-c:v", vcodec,
          "-c:a", acodec,
          "-shortest",
          "-crf", "18",
          # "-avoid_negative_ts", "1",
          out_path
        ]
        subprocess_call(cmd)
        return out_path
